@@ -6,8 +6,10 @@ from types import MethodType
 from tqdm import tqdm
 
 import whisper
-from whisper.audio import SAMPLE_RATE, N_FRAMES, HOP_LENGTH, N_SAMPLES, N_SAMPLES_PER_TOKEN, \
+from whisper.audio import (
+    SAMPLE_RATE, N_FRAMES, HOP_LENGTH, N_SAMPLES, N_SAMPLES_PER_TOKEN, TOKENS_PER_SECOND, FRAMES_PER_SECOND, N_FFT,
     pad_or_trim, log_mel_spectrogram
+)
 from whisper.utils import exact_div, format_timestamp, make_safe
 from whisper.tokenizer import get_tokenizer, LANGUAGES, TO_LANGUAGE_CODE
 from whisper.decoding import DecodingOptions, DecodingResult
@@ -57,6 +59,7 @@ def transcribe_stable(
         only_voice_freq: bool = False,
         prepend_punctuations: str = "\"'“¿([{-",
         append_punctuations: str = "\"'.。,，!！?？:：”)]}、",
+        mel_first: bool = False,
         **decode_options) \
         -> WhisperResult:
     """
@@ -104,9 +107,7 @@ def transcribe_stable(
         Disabling this will prevent segments from splitting/merging properly.
 
     regroup: bool
-        Split segments at periods or question marks,
-        then split segments if the gap between two segments more than 0.5 seconds,
-        then merge segments if the gap between two segments more than 0.15 seconds. (Default: True)
+        Regroup all words into segments with more natural boundaries.(Default: True)
         Ignored if [word_timestamps]=False.
 
     ts_num: int
@@ -168,6 +169,10 @@ def transcribe_stable(
     append_punctuations: str
         Punctuations to append to previous word (Default: .。,，!！?？:：”)]}、)
 
+    mel_first: bool
+        Process entire audio track into log-Mel spectrogram first instead in chunks. (Default: False)
+        Used if odd behavior seen in stable-ts but not in whisper, but use significantly more memory for long audio.
+
     decode_options: dict
         Keyword arguments to construct `DecodingOptions` instances
 
@@ -227,13 +232,16 @@ def transcribe_stable(
     if only_voice_freq:
         from .audio import voice_freq_filter
         audio = voice_freq_filter(audio, curr_sr)
+    sample_padding = int(N_FFT//2)+1
+    whole_mel = log_mel_spectrogram(audio, padding=sample_padding) if mel_first else None
 
     if decode_options.get("language", None) is None:
         if verbose:
             print("Detecting language using up to the first 30 seconds. Use `--language` to specify the language")
-        segment = pad_or_trim(log_mel_spectrogram(audio[..., :N_SAMPLES]),
-                              N_FRAMES).to(device=model.device, dtype=dtype)
-        _, probs = model.detect_language(segment)
+        mel_segment = log_mel_spectrogram(audio[..., :N_SAMPLES], padding=sample_padding) \
+            if whole_mel is None else whole_mel[..., :N_FRAMES]
+        mel_segment = pad_or_trim(mel_segment, N_FRAMES).to(device=model.device, dtype=dtype)
+        _, probs = model.detect_language(mel_segment)
         decode_options["language"] = max(probs, key=probs.get)
         print(f"Detected language: {LANGUAGES[decode_options['language']]}")
 
@@ -309,7 +317,7 @@ def transcribe_stable(
         tokens = tokens.tolist()
         text_tokens = [token for token in tokens if token < tokenizer.eot]
         return {
-            "seek": round(seek_sample / SAMPLE_RATE, 4),  # units in seconds
+            "seek": round(seek_sample / SAMPLE_RATE, 3),  # units in seconds
             "start": start,
             "end": end,
             "text": tokenizer.decode(text_tokens),
@@ -322,6 +330,7 @@ def transcribe_stable(
 
     total_samples = audio.shape[-1]
     total_duration = round(total_samples / curr_sr, 2)
+    n_samples_per_frame = exact_div(N_SAMPLES_PER_TOKEN * TOKENS_PER_SECOND,  FRAMES_PER_SECOND)
 
     silence_timing = None
     if suppress_silence and vad:
@@ -345,12 +354,18 @@ def transcribe_stable(
             update_pbar()
 
         while seek_sample < audio.shape[-1]:
-            audio_segment = audio[seek_sample:seek_sample + N_SAMPLES]
+            seek_sample_end = seek_sample + N_SAMPLES
+            audio_segment = audio[seek_sample:seek_sample_end]
             time_offset = seek_sample / SAMPLE_RATE
             segment_samples = min(N_SAMPLES, total_samples - seek_sample)
             segment_duration = segment_samples / SAMPLE_RATE
 
-            mel_segment = log_mel_spectrogram(audio_segment)
+            mel_segment = (
+                log_mel_spectrogram(audio_segment, padding=sample_padding)
+                if whole_mel is None else
+                whole_mel[..., round(seek_sample/n_samples_per_frame): round(seek_sample_end/n_samples_per_frame)]
+            )
+
             mel_segment = pad_or_trim(mel_segment, N_FRAMES).to(device=model.device, dtype=dtype)
 
             segment_silence_timing = None
@@ -372,12 +387,11 @@ def transcribe_stable(
                         silence_timing = (silence_timing[0][mn:], silence_timing[1][mn:])
 
                 if ts_token_mask is not None:
+                    if ts_token_mask.all():  # segment is silent
+                        fast_forward()
+                        continue
                     ts_token_mask = pad_or_trim(ts_token_mask, 1501)
 
-                if segment_silence_timing is None:
-                    fast_forward()
-                    continue
-
             decode_options["prompt"] = all_tokens[prompt_reset_since:]
             result: DecodingResult = decode_with_fallback(mel_segment, ts_token_mask=ts_token_mask)
             tokens = torch.tensor(result.tokens)
@@ -417,8 +431,8 @@ def transcribe_stable(
                     )
                     current_segments.append(
                         new_segment(
-                            start=round(time_offset + start_timestamp_pos * time_precision, 2),
-                            end=round(time_offset + end_timestamp_pos * time_precision, 2),
+                            start=round(time_offset + start_timestamp_pos * time_precision, 3),
+                            end=round(time_offset + end_timestamp_pos * time_precision, 3),
                             tokens=sliced_tokens,
                             result=result,
                         )
@@ -446,8 +460,8 @@ def transcribe_stable(
 
                 current_segments.append(
                     new_segment(
-                        start=round(time_offset, 2),
-                        end=round(time_offset + duration, 2),
+                        start=round(time_offset, 3),
+                        end=round(time_offset + duration, 3),
                         tokens=tokens,
                         result=result,
                     )
@@ -487,7 +501,7 @@ def transcribe_stable(
                         Segment(**segment)
                         .suppress_silence(
                             *segment_silence_timing,
-                            min_word_dur,
+                            min_word_dur=min_word_dur,
                             word_level=suppress_word_ts
                         )
                         .to_dict()
@@ -665,9 +679,7 @@ def cli():
                              "disabling this will prevent segments from splitting/merging properly.")
 
     parser.add_argument("--regroup", type=str2bool, default=True,
-                        help="split segments at periods or question marks;"
-                             "then split segments if the gap between two segments more than 0.5 seconds;"
-                             "then merge segments if the gap between two segments more than 0.15 seconds;"
+                        help="regroup all words into segments with more natural boundaries;"
                              "ignored if [word_timestamps]=False.")
 
     parser.add_argument('--ts_num', type=int, default=0,
